<?php
/**
 * API access to Weezo
 * Request /api.php?help for help on arguments
 * 	apiVersion: API version (1)
 * 	apiClientName: name of client API (ex.: iPhoneApp, androidApp...)
 * 	responseFormat: "XML" or "JSON"
 * 	action: requested action
 *
 * Optional
 * 	lng: ISO 639-1 formated language code for response (2 chars)
 * 	includeRequest: set to get a request node in response, containing all request arguments
 */
define('API_LOADED', true); // Used to indicate to other scripts we're in API mode
define('API_VERSION', 0.1); // API version
define('API_BEAUTIFY_CR', "\n"); // Beautified cr / lb
require_once(cfAppDocRoot().'/login.php'); // Include login.php for misc. login functions

/**
 * @desc Class used for request actions definition (help text, needs login, minimal arguments...)
 *
 */
class apiRequestAction{
	private $actionName; // Action name
	private $functionName; // Name of apiRequest function that will process this action
	private $helpText; // Action description
	private $minimalArgs=array(); // array of required args: name => 'help text'
	private $optionalArgs=array(); // array of optional args: name => 'help text'
	private $requiresLogin=true; // false if user doesn't need to be authenticated to perform request

	/**
	 * @desc Constructor
	 *
	 * @param string $actionName: Action name
	 * @param string $functionName: Name of apiRequest function that will process this action
	 */
	function __construct($actionName,$functionName){
		$this->actionName=$actionName;
		$this->functionName=$functionName;
	}

	/**
	 * @desc Add a line of text to help
	 *
	 * @param string $helpText
	 */
	function setHelp($helpText){
		$this->helpText.=(($this->helpText)?"\n":'').$helpText;
	}

	/**
	 * @desc Get help text
	 *
	 * @return string
	 */
	function getHelp(){
		return $this->helpText;
	}

	/**
	 * @desc Get 1st line of help text
	 *
	 * @return string
	 */
	function getShortHelp(){
		if(strpos($this->helpText,"\n")) return substr($this->helpText,0,strpos($this->helpText,"\n"));
		return $this->helpText;
	}

	/**
	 * @desc Get all but 1st line of help text (should contain response info)
	 *
	 * @return string
	 */
	function getResponsetHelp(){
		if(strpos($this->helpText,"\n")) return substr($this->helpText,strpos($this->helpText,"\n")+1);
		return '';
	}

	/**
	 * @desc Set minimal arguments and help string
	 *
	 * @param $argName: name of arg
	 * @param $helpText: help text for this arg
	 */
	function addMinimalArgs($argName,$helpText){
		$this->minimalArgs[$argName]=$helpText;
	}
	/**
	 * @desc Get minimal arguments
	 *
	 * @param array $args
	 */
	function getMinimalArgs(){
		return $this->minimalArgs;
	}


	/**
	 * @desc Set optional arguments and help string
	 *
	 * @param $argName: name of arg
	 * @param $helpText: help text for this arg
	 */
	function addOptionalArgs($argName,$helpText){
		$this->optionalArgs[$argName]=$helpText;
	}
	/**
	 * @desc Get optional arguments
	 *
	 * @param array $args
	 */
	function getoptionalArgs($args){
		$this->optionalArgs=$args;
	}

	/**
	 * @desc Set / Get if action requires to be logged
	 *
	 * @param boolean $requiresLogin: true or false to set, empty to get
	 */
	function requiresLogin($requiresLogin=null){
		if($requiresLogin===null) return $this->requiresLogin;
		else $this->requiresLogin=$requiresLogin;
	}

	/**
	 * @desc call this action/function
	 *
	 */
	function call($apiRequest){
		call_user_func(array($apiRequest,$this->functionName));
	}
}

/**
 * @desc API response object
 *
 */
class apiResponse{
	private $format;
	private $responseData=array();
	private $beautified=false;
	private $responseAllowedFormats=array('XML','JSON');

	/**
	 * @call this function will result in a tabulated and line-returned result
	 * @param $beautify: true to beautify, 'HTML' to beautify for HTML output
	 */
	public function beautify($beautify=true){
		$this->beautified=$beautify;
	}

	/**
	 * @desc Constructor
	 *
	 * @param string $format: response format (see $responseAllowedFormats for valid formats)
	 */
	public function __construct($format) {
		if(in_array($format,$this->responseAllowedFormats)) $this->format=$format;
	}

	/**
	 * @desc Return true if apiResponse has been correctly created (i.e. format is ok)
	 *
	 */
	public function valid(){
		return ($this->format)?true:false;
	}

	/**
	 * @desc Add a response node
	 *
	 * @param string $key: node name/key
	 * @param mixed $data: string or array of nodes
	 * @param boolean $isBinary: true if string contains binary data
	 */
	public function add($key,$data,$isBinary=false){
		// Add an array of data
		if(is_array($data)) $this->responseData[$key]=$data;

		// Add a single data item
		else $this->responseData[$key]=array('value'=>$data,'binary'=>$isBinary);
	}

	/**
	 * @desc Return an error to client
	 *
	 * @param string $errorId: id of error (see above)
	 * @param string $caption: utf8 encoded error message
	 * @param boolean $exit: exit after sending error if true
	 */
	public function returnError($errorId,$caption,$exit=true){
		$this->add('error',array('id'=>$errorId,'message'=>$caption));
		$this->send($exit);
	}

	/**
	 * @desc Add a node to response
	 *
	 * @param array $node: key=>value (value is a string or a node)
	 * @return mixed
	 */
	private function formatedNode($node,$level=0){
		$output='';

		$tab=($this->beautified)?str_pad('',$level+1,"\t"):'';
		$cr=($this->beautified)?API_BEAUTIFY_CR:'';
		switch ($this->format){
			case 'XML':
				foreach ($node as $k=>$v){
					// If node is an array of nodes, add subnodes
					if(is_array($v) && !(count($v)==2 && isset($v['binary']) && isset($v['value']))){
						if(!count($v)) $output.=$tab.'<'.$k."/>".$cr;
						else $output.=$tab.'<'.$k.">".$cr.$this->formatedNode($v,$level+1).$tab.'</'.$k.">".$cr;
					}
					// Else, add node
					else{
						if(!is_array($v)) $v=array('value'=>$v,'binary'=>false);

						$output.=$tab.'<'.$k;

						if(!$v['value']) $output.="/>".$cr;
						else{
							// Binary data
							if($v['binary']) $output.=' b64="b64">'.base64_encode($v['value']);
							// Text data
							else $output.='>'.str_replace('<','&lt;',$v['value']);

							$output.='</'.$k.">".$cr;
						}
					}
				}
				break;
			case 'JSON':
				$nb=count($node);
				$i=0;
				foreach ($node as $k=>$v){
					$output.=$tab.'"'.$k.'":';

					// If node is an array of nodes, add subnodes
					if(is_array($v) && !(count($v)==2 && isset($v['binary']) && isset($v['value']))){
						$output.="{".$cr;
						$output.=$this->formatedNode($v,$level+1);
						$output.=$tab.'}';
					}
					// Else, add node
					else{
						if(!is_array($v)) $v=array('value'=>$v,'binary'=>false);

						// Binary data
						if($v['binary']) $output.='"Content-Transfer-Encoding: base64\\\\'.base64_encode($v['value']).'"';
						// Text data
						else $output.='"'.addslashes($v['value']).'"';
					}
					$i++;
					if($i==$nb) $output.=$cr; else $output.=",".$cr;
				}
				break;
		}
		return $output;
	}

	/**
	 * @desc Send response
	 *
	 * @param boolean $exit: exit after if set to true
	 */
	public function send($exit=true){
		$output='';
		switch ($this->format){
			case 'XML':
				$output.='<?xml version="1.0" encoding="utf-8"?>'."\n";
				$output.="<response>".(($this->beautified)?"\n":'');
				$output.=$this->formatedNode($this->responseData);
				$output.='</response>';
				break;

			case 'JSON':
				$output.="{".(($this->beautified)?"\n":'');
				$output.=$this->formatedNode($this->responseData);
				$output.='}';
				break;
		}
		// HTML beautifying
		if($this->beautified==='HTML') $output=str_replace(API_BEAUTIFY_CR,'<br>',str_replace("\t",'&nbsp;&nbsp;&nbsp;&nbsp;',htmlentities($output)));
		echo $output;

		if ($exit) die();
	}
}

/**
 * @desc API request object
 *
 */
class apiRequest{
	private $format=false; // Request format
	private $response; // Response object
	private $responseFormat=false; // Response format (XML)
	private $args; // Request arguments
	private $lng; // Request language
	public	$accountName; // weezo.net account name
	private $user; // weezo.net user object (array)
	private $actions; // Array of apiRequestAction objects, defining all available actions

	/**
	 * List of all available action name=>function name. Every action must be listed here!
	 */
	private $actionsList=array(
		'help'=>'showHelp', // Show API general help
		'getAPIVersion'=>'getAPIVersion', // Get this API's version
		'listActions'=>'listActions', // List all available actions (except for resources)
		'getGroups'=>'getUsersList', // List all accessible user groups
		'login'=>'login' // Persistent login
	);

	/**
	 * @desc Create request object, detect format from input
	 *
	 */
	function __construct($args=false) {
		
		/******************************************************************************************************************************
		 * GET/SET request and response formats
		 *****************************************************************************************************************************/
		
		
		// Set default response format
		$default=array(
			'apiClientName'=>'unknown', // API client name
			'lng'=>'en', // Response language
			'format'=>'XML' // Response format
		);
		
		// Internal API request
		if($args){
			$this->format='INTERNAL';
			$this->args=$args;
			$this->args['format']='INTERNAL';
			$this->args['apiClientName']='INTERNAL';
		}

		// POST request
		elseif(isset($_POST['action']) || isset($_POST['help'])) {
			$this->format='POST';
			$this->args=$_POST+$default;
			if(!isset($this->args['action']) && isset($_POST['help'])) $this->args['action']='help';
		}

		// GET request
		elseif(isset($_GET['action']) || isset($_GET['help'])) {
			$this->format='GET';
			$this->args=$_GET+$default;
			if(!isset($this->args['action']) && isset($_GET['help'])) $this->args['action']='help';
		}
		else $this->args=$default;

		// Set response format
		$this->responseFormat=$this->args['format'];
		$this->response=new apiResponse($this->responseFormat);
		if(isset($this->args['beautify'])) $this->response->beautify($this->args['beautify']);

		// Verify response format is supported
		if(!$this->response->valid()) die('Incorrect response format');

		// If request format couldn't be set, stop here
		if(!$this->format) $this->response->returnError('unknownFormat','Incorrect request format. GET or POST action=help or help to get more info');

		// Set response language into $_GET so cfCaption functions can be used
		$_GET['lng']=$this->args['lng'];
		
		
		/******************************************************************************************************************************
		 * Check access rights and minimal parameters
		 *****************************************************************************************************************************/

		// Add original request arguments to response
		if($this->response && isset($this->args['includeRequest'])) {
			$tmp=$this->args;
			unset($tmp['includeRequest']);
			$this->response->add('request',$tmp);
			unset($tmp);
		}

		// Check for minimal parameters
		if(!isset($this->args['action'])) $this->response->returnError('incorrectParameters','Incorrect API parameters: missing action command. Use "help" for more info');

		// Check if client IP is allowed to connect to server
		if(cfIPBanned() || !cfIPFilterCheck()) {
			sleep(cfGGetVar('waitTimeBetweenLoginAttemtps'));
			$this->response->returnError('loginForbiddenAccess',cfCaption('loginForbiddenAccess'));
		}

		
		/*****************************************************************************************************************************
		 * Check action is valid
		 *****************************************************************************************************************************/
		
		// General-level request
		if(!@$this->args['resId']){

			// Build actions list
			foreach ($this->actionsList as $actionName=>$functionName) {
				$this->actions[$actionName]=new apiRequestAction($actionName,$functionName);
				// Let funtion fill help, requires login, parameters...
				call_user_func(array(__CLASS__,$functionName),$this->actions[$actionName]);
			}
			
			// Verify action is actually in available  actions list
			if(!isset($this->actionsList[$this->args['action']]))
				$this->response->returnError('incorrectAction','Unknown action. Use action=listActions for actions list');
		}
		// Resource-level request
		else{
			// 
			$configs=WEnv::resConfigs();
			if(0) $config=new WResConfig(); // Zend IDE WA
			
			// Inexistant resource
			if(!isset($configs[$this->args['resId']])) $this->response->returnError('unknownResource','Unknown or forbidden resource');
			
			$config=$configs[$this->args['resId']];
			
			// Resource has no API access
			if(!$config->API()) $this->response->returnError('APIUnsupported','This resource has no API access');
			
			// Include resource's API script
			$apiEntryPointName=false;
			require_once($config->API());
			if(!$apiEntryPointName || !function_exists($apiEntryPointName)) $this->response->returnError('APIUnsupported','This resource has no API access');
			
			
		}
		

		/******************************************************************************************************************************
		 * Internal API call: directly process action (no need to add a call to process())
		 *****************************************************************************************************************************/
		if($this->format=='INTERNAL'){
			$this->response->process();
		}
	}

	
	
	/**
	 * @desc Process command (action)
	 *
	 */
	public function process(){
		$actionName=$this->args['action'];
		$action=$this->actions[$actionName]; if(0) $action=new apiRequestAction(); // Zend IDE WA

		// Check if action is authorized depending on user state (logged / not logged)
		if($action->requiresLogin() && !$this->isLogged()) $this->response->returnError('unauthorizedAction','Action not authorized');

		// Help on a specific action requested
		if(isset($this->args['help'])){
			$this->response->add('description',$action->getShortHelp());
			$this->response->add('mandatory_arguments',$action->getMinimalArgs());
			$this->response->add('optional_arguments',$action->getMinimalArgs());
			$this->response->add('response_arguments',$action->getResponsetHelp());
			// Don't process request, directly send response
			return $this->response->send();
		}

		// Check minimal arguments
		foreach ($action->getMinimalArgs() as $argName=>$helpText) if(!isset($this->args[$argName])){
			$this->response->returnError('missingArgument','Missing "'.$argName.'" argument. Use help argument to get help on '.$actionName);
		}

		/**
		 * Process action
		 */
		$action->call($this);

		// Send response (function result is returned for INTERNAL requests)
		return $this->response->send();
	}

	/**
	 * @desc Get this very API version
	 *
	 */
	public function getAPIVersion($actionDefinition=false){
		// Action definition. Called on apiRequest construction
		if($actionDefinition){
			$actionDefinition->setHelp('Get API version.');
			$actionDefinition->setHelp('Response:');
			$actionDefinition->setHelp('	v: version');
			$actionDefinition->requiresLogin(false);
			return;
		}
		$this->response->add('v', API_VERSION);
	}

	/**
	 * @desc Show general help
	 *
	 */
	public function showHelp($actionDefinition=false){
		// Action definition. Called on apiRequest construction
		if($actionDefinition){
			$actionDefinition->setHelp('Mandatory parameters:');
			$actionDefinition->setHelp('	action: case-sensitive API action (mandatory). use action=listActions to get a list of actions');
			$actionDefinition->setHelp('');
			$actionDefinition->setHelp('Optional parameters');
			$actionDefinition->setHelp('	help: display help and parameters for a given action');
			$actionDefinition->setHelp('	format: response format: "JSON" or "XML"');
			$actionDefinition->setHelp('	resId: use this parameter with a valid resource id to pass the action to this resource');
			$actionDefinition->setHelp('	filterInclude: comma separated list of exclusively kept parameters in response');
			$actionDefinition->setHelp('	filterExclude: comma separated list of parameters removed from response');
			$actionDefinition->setHelp('	beautify: set this option to include tabs and line breaks in response. use "HTML" value to use HTML beautified format');
			$actionDefinition->requiresLogin(false);
			return;
		}
		$this->response->add('help', $this->actions['help']->getHelp());
	}

	/**
	 * @desc List all available actions (commands)
	 *
	 */
	public function listActions($actionDefinition=false){
		// Action definition. Called on apiRequest construction
		if($actionDefinition){
			$actionDefinition->setHelp('List all available API actions');
			$actionDefinition->requiresLogin(false);
			return;
		}
		foreach ($this->actions as $actionName=>$actionDefinition) {
			if($actionName!=='listActions' && $actionName!=='help')
			$this->response->add($actionName,$actionDefinition->getShortHelp());
		}
	}

	/**
	 * @desc retreive users list
	 *
	 */
	public function getUsersList(&$actionDefinition=false){
		if(0) {$actionDefinition=new apiRequestAction(); $user=new WUserConfig();} // Used for IDE autocompletition

		// Action definition. Called on apiRequest construction
		if($actionDefinition){
			$actionDefinition->setHelp('Get a list of all accessible user groups');
			$actionDefinition->setHelp('Response:');
			$actionDefinition->setHelp('	id: user unique ID');
			$actionDefinition->setHelp('	name: UTF8 encoded user name');
			$actionDefinition->setHelp('	accountType: account type (singleUser, named, anonymous) (named account login requires pseudo)');
			$actionDefinition->setHelp('	hasPassword: 1 if user has password, else 0');
			$actionDefinition->setHelp('	iconURL: URL to icon (size of image can be set with iconSize command)');

			$actionDefinition->requiresLogin(false);
			
			$actionDefinition->addOptionalArgs('iconSize','width & height of icon image');
			return;
		}

		$result=array();

		// Parse include / exclude filters
		$excluded=(isset($this->args['filterExclude']))?explode(',',$this->args['filterExclude']):false;
		$included=(isset($this->args['filterInclude']))?explode(',',$this->args['filterInclude']):false;
		$nb=0;

		// Browse allowed users accounts
		foreach (WEnv::userConfigs() as $id=>$user){
			// Check user is valid, not hidden, not invisible (system), activated, and not forbidden to this ip
			if($user->isValid() && !$user->getVar('hidden') && $user->getVar('activated') && !$user->isInvisible() && cfIPFilterCheck($user->getData())){
				// User id
				$result['user'.$nb]['id']=$id;

				// User name
				$result['user'.$nb]['name']=cfUTF8Encode($user->getVar('name'));

				// User name
				$result['user'.$nb]['accountType']=$user->getVar('accountType');

				// User password
				$result['user'.$nb]['hasPassword']=($user->getVar('authenticationMethod')=='password')?1:0;

				// User password hint
				if($result['user'.$nb]['hasPassword'] && $user->getVar('hint'))
					$result['user'.$nb]['passwordHint']=cfUTF8Encode($user->getVar('hint'));

				// User icon
				$result['user'.$nb]['iconURL']='/gfx/icons/'.$user->getVar('icon');
				if(isset($this->args['iconSize'])) $result['user'.$nb]['iconURL']=cfExtGFX('/gfx/icons/'.$user->getVar('icon'),$this->args['iconSize'],$this->args['iconSize']);

				// Keep only requested values
				if($excluded){
					foreach ($result['user'.$nb] as $k=>$v) if(in_array($k,$excluded)) unset($result['user'.$nb][$k]);
				}
				if($included){
					foreach ($result['user'.$nb] as $k=>$v) if(!in_array($k,$included)) unset($result['user'.$nb][$k]);
				}

				$nb++;
			}
		}
		$this->response->add('users',$result);
	}

	/**
	 * @desc get RSA Public key used for login
	 *
	 */
	public function getRSAPK($actionDefinition=false){
		// Action definition. Called on apiRequest construction
		if($actionDefinition){
			$actionDefinition->setHelp('Get RSA public key used for login');
			$actionDefinition->setHelp('Response:');
			$actionDefinition->setHelp('	maxDigits');
			$actionDefinition->setHelp('	modulo');
			$actionDefinition->setHelp('	publicExponent');
			$actionDefinition->setHelp('	keyLength');
			
			$actionDefinition->requiresLogin(false);
			return;
		}

		// Load RSA keypair
		$RSAKeyPair = cfRSALoadKeyPair();

		// If RSA keypair successfully loaded, return public key
		if(isset($RSAKeyPair['maxDigits']) && isset($RSAKeyPair['modulo']) && isset($RSAKeyPair['publicExponent'])){
			die('RSAPK/'.$RSAKeyPair['maxDigits'].'/'.$RSAKeyPair['modulo'].'/'.$RSAKeyPair['publicExponent'].'/'.$RSAKeyPair['keyLength']);
			$this->response->add('maxDigits',$RSAKeyPair['maxDigits']);
			$this->response->add('modulo',$RSAKeyPair['modulo']);
			$this->response->add('publicExponent',$RSAKeyPair['publicExponent']);
			$this->response->add('keyLength',$RSAKeyPair['keyLength']);
		}
		else $this->response->returnError('internalErrorNoRSAPK','Internal error: missing RSA key pair');
	}

	/**
	 * @desc Log user in
	 *
	 */
	public function login($actionDefinition=false){
		// Action definition. Called on apiRequest construction
		if($actionDefinition){
			$actionDefinition->setHelp('Persistent user login.');
			$actionDefinition->setHelp('Response:');
			$actionDefinition->setHelp('	error: "wrongIdOrPassword", "needPseudo" (user (+password) ok but a pseudo is required. user & password must be resent with pseudo)');
			$actionDefinition->setHelp('	or');
			$actionDefinition->setHelp('	sessionId: id of session to use');
			
			$actionDefinition->requiresLogin(false);

			$actionDefinition->addOptionalArgs('groupId','ID of group to connect to');
			$actionDefinition->addOptionalArgs('groupName','Name of group to connect to');
			$actionDefinition->addOptionalArgs('password','Group or direct resource access password');
			$actionDefinition->addOptionalArgs('pseudo','User pseudo');

			$actionDefinition->addOptionalArgs('publishToken','Access token to a published resource');
			$actionDefinition->addOptionalArgs('directResourceAccessToken','Access token to direct access a resource (may require a password)');
			return;
		}

		// Direct resource access through directResourceAccessToken
		if(@$this->args['directResourceAccessToken']) {
			if(!directResourceAccessTokenLogin($this->args['directResourceAccessToken'],@$this->args['password'],true))
				$this->response->returnError('wrongIdOrPassword',cfCaption('loginBadPasswordOrAccount'));

			// Store direct resource access token into $_GET so it's recognized by userLoad function
			$_GET['directResourceAccessToken']=$this->args['directResourceAccessToken'];

			$wSessId=userLoad('standalone',false,true);
		}

		// Regular ID/Password login
		elseif(isset($this->args['groupId']) || isset($this->args['groupName'])){
			$users=WEnv::userConfigs();

			// User id passed
			if(isset($this->args['groupId'])){
				if(!isset($users[$this->args['groupId']])) $this->response->returnError('wrongIdOrPassword',cfCaption('loginBadPasswordOrAccount'));
				$user=$users[$this->args['groupId']];
			}
			// User name passed
			else{
				foreach ($users as $id=>$user) if($user->getVar('name')==cfUTF8Decode($this->args['groupName'])){
					$found=true;
					break;
				}
				if(!isset($found)) $this->response->returnError('wrongIdOrPassword',cfCaption('loginBadPasswordOrAccount'));
			}
			if(0) $user=new WUserConfig(); // Zend IDE WA

			// Check password
			if($user->getVar('authenticationMethod')==='password' && !cfHashedPasswordCheck(@$this->args['password'],$user->getVar('password')))
				$this->response->returnError('wrongIdOrPassword',cfCaption('loginBadPasswordOrAccount'));

			// Facebook login (unsupported yet)
			elseif($user->getVar('authenticationMethod')==='facebook')
				$this->response->returnError('unsupportedMethod','Facebook API login not supported');

			// If account requires a pseudo
			if($user->getVar('accountType')==='named'){
				// If pseudo sent, store it into $_ENV var so it's integrated by userLoad function
				if (@$this->args['pseudo']) $_ENV['userPseudo']=$this->args['pseudo'];
				else $this->response->returnError('needPseudo',cfCaption('userTypeName'));
			}

			// Do log in
			$wSessId=userLoad($user->id(),false,true);
		}

		// Login Ok: return session Id
		if(@$wSessId){
			$this->response->add('result','ok');
			$this->response->add('WSESSID',$wSessId);
		}
	}
}


// If api.php is directly called by client, process request
if($_SERVER['SCRIPT_NAME']=='/api.php'){
	
	// Create request object
	$req=new apiRequest();

	// Process request
	$req->process();
}

?>